@@ -41,6 +41,7 @@
 #include "fcitx-config/xdg.h"
 #include "fcitx-utils/log.h"
 #include "module/spell/fcitx-spell.h"
+#include "module/freedesktop-notify/fcitx-freedesktop-notify.h"
 
 #include "keyboard.h"
 #if defined(ENABLE_LIBXML2)
@@ -140,8 +141,14 @@ void Ucs4ToUtf8(iconv_t conv, uint32_t ucs4, char* utf8)
 
 CONFIG_DESC_DEFINE(GetKeyboardConfigDesc, "fcitx-keyboard.desc")
 
-FCITX_DEFINE_PLUGIN(fcitx_keyboard, ime, FcitxIMClass) = {
+FCITX_DEFINE_PLUGIN(fcitx_keyboard, ime2, FcitxIMClass2) = {
     FcitxKeyboardCreate,
+    NULL,
+    FcitxKeyboardReloadConfig,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
     NULL
 };
 
@@ -154,7 +161,7 @@ static const char* FcitxKeyboardGetPastStream(void* arg)
     char* surrounding = NULL;
     unsigned int cursor = 0, anchor = 0;
     if (FcitxInstanceGetSurroundingText(keyboard->owner, FcitxInstanceGetCurrentIC(keyboard->owner), &surrounding, &cursor, &anchor)) {
-        int len = cursor + strlen(keyboard->buffer) + 1;
+        int len = cursor + strlen(keyboard->buffer[0]) + 1;
         if (keyboard->lastLength < len) {
             free(keyboard->tempBuffer);
             while (keyboard->lastLength < len)
@@ -179,7 +186,7 @@ static const char* FcitxKeyboardGetPastStream(void* arg)
     }
     else
 #endif
-    return keyboard->buffer;
+    return keyboard->buffer[0];
 }
 
 static const char* FcitxKeyboardGetFutureStream(void* arg)
@@ -261,7 +268,7 @@ void FcitxKeyboardLayoutCreate(FcitxKeyboard* keyboard,
     iface.DoInput = FcitxKeyboardDoInput;
     iface.GetCandWords = FcitxKeyboardGetCandWords;
     iface.Save = FcitxKeyboardSave;
-    iface.ReloadConfig = FcitxKeyboardReloadConfig;
+    iface.ReloadConfig = NULL;
     iface.OnClose = FcitxKeyboardOnClose;
 
     FcitxInstanceRegisterIMv2(
@@ -283,6 +290,70 @@ void* SimpleCopy(void* arg, void* dest, void* src)
     return src;
 }
 
+#if defined(ENABLE_LIBXML2)
+static const char* FindBestLanguage(FcitxIsoCodes* isocodes, const char* hint, UT_array* languages)
+{
+    const char* bestLang = NULL;
+
+    /* score:
+     * 1 -> first one
+     * 2 -> match 2
+     * 3 -> match three
+     */
+    FcitxIsoCodes639Entry* bestEntry = NULL;
+    int bestScore = 0;
+    utarray_foreach(plang, languages, char*) {
+        FcitxIsoCodes639Entry* entry = FcitxIsoCodesGetEntry(isocodes, *plang);
+        if (!entry) {
+            continue;
+        }
+
+        const char* lang = entry->iso_639_1_code;
+        if (!lang) {
+            lang = entry->iso_639_2T_code;
+        }
+
+        if (!lang) {
+            lang = entry->iso_639_2B_code;
+        }
+
+        if (!lang) {
+            continue;
+        }
+
+        size_t len = strlen(lang);
+        if (len != 2 && len != 3) {
+            continue;
+        }
+
+        int score = 1;
+        while (len >= 2) {
+            if (strncasecmp(hint, lang, len) == 0) {
+                score = len;
+                break;
+            }
+
+            len --;
+        }
+
+        if (bestScore < score) {
+            bestEntry = entry;
+            bestScore = score;
+        }
+    }
+    if (bestEntry) {
+        bestLang = bestEntry->iso_639_1_code;
+        if (!bestLang) {
+            bestLang = bestEntry->iso_639_2T_code;
+        }
+        if (!bestLang) {
+            bestLang = bestEntry->iso_639_2B_code;
+        }
+    }
+    return bestLang;
+}
+#endif
+
 void* FcitxKeyboardCreate(FcitxInstance* instance)
 {
     FcitxKeyboard* keyboard = fcitx_utils_malloc0(sizeof(FcitxKeyboard));
@@ -339,21 +410,9 @@ void* FcitxKeyboardCreate(FcitxInstance* instance)
         }
 
         FcitxIsoCodes* isocodes = FcitxXkbReadIsoCodes(ISOCODES_ISO639_XML, ISOCODES_ISO3166_XML);
-        FcitxXkbLayoutInfo* layoutInfo;
-        for (layoutInfo = (FcitxXkbLayoutInfo*) utarray_front(rules->layoutInfos);
-            layoutInfo != NULL;
-            layoutInfo = (FcitxXkbLayoutInfo*) utarray_next(rules->layoutInfos, layoutInfo))
-        {
+        utarray_foreach(layoutInfo, rules->layoutInfos, FcitxXkbLayoutInfo) {
             {
-                char** plang = NULL;
-                plang = (char**) utarray_front(layoutInfo->languages);
-                char* lang = NULL;
-                if (plang) {
-                    FcitxIsoCodes639Entry* entry = FcitxIsoCodesGetEntry(isocodes, *plang);
-                    if (entry) {
-                        lang = entry->iso_639_1_code;
-                    }
-                }
+                const char* lang = FindBestLanguage(isocodes, layoutInfo->description, layoutInfo->languages);
                 char *description;
                 fcitx_utils_alloc_cat_str(description, _("Keyboard"), " - ",
                                           dgettext("xkeyboard-config",
@@ -369,17 +428,8 @@ void* FcitxKeyboardCreate(FcitxInstance* instance)
                 variantInfo != NULL;
                 variantInfo = (FcitxXkbVariantInfo*) utarray_next(layoutInfo->variantInfos, variantInfo))
             {
-                char** plang = NULL;
-                plang = (char**) utarray_front(variantInfo->languages);
-                if (!plang)
-                    plang = (char**) utarray_front(layoutInfo->languages);
-                char* lang = NULL;
-                if (plang) {
-                    FcitxIsoCodes639Entry* entry = FcitxIsoCodesGetEntry(isocodes, *plang);
-                    if (entry) {
-                        lang = entry->iso_639_1_code;
-                    }
-                }
+                const char* lang = FindBestLanguage(isocodes, layoutInfo->description,
+                                                    utarray_len(variantInfo->languages) > 0 ? variantInfo->languages : layoutInfo->languages);
                 char *description;
                 fcitx_utils_alloc_cat_str(description, _("Keyboard"), " - ",
                                           dgettext("xkeyboard-config",
@@ -493,6 +543,8 @@ boolean FcitxKeyboardInit(void *arg)
                             CONTEXT_DISABLE_QUICKPHRASE, &flag);
     FcitxInstanceSetContext(layout->owner->owner,
                             CONTEXT_DISABLE_FULLWIDTH, &flag);
+    FcitxInstanceSetContext(layout->owner->owner,
+                            CONTEXT_DISABLE_AUTO_FIRST_CANDIDATE_HIGHTLIGHT, &flag);
     if (layout->variantString) {
         char *string;
         fcitx_utils_alloc_cat_str(string, layout->layoutString, ",",
@@ -510,9 +562,10 @@ boolean FcitxKeyboardInit(void *arg)
 
 void FcitxKeyboardCommitBuffer(FcitxKeyboard* keyboard)
 {
-    if (keyboard->buffer[0]) {
-        FcitxInstanceCommitString(keyboard->owner, FcitxInstanceGetCurrentIC(keyboard->owner),
-                                    keyboard->buffer);
+    if (keyboard->buffer[0][0]) {
+        FcitxInstanceCommitString(keyboard->owner,
+                                  FcitxInstanceGetCurrentIC(keyboard->owner),
+                                  keyboard->buffer[0]);
         FcitxKeyboardResetIM(keyboard);
     }
 }
@@ -521,7 +574,8 @@ void  FcitxKeyboardResetIM(void *arg)
 {
     FcitxKeyboardLayout *layout = (FcitxKeyboardLayout*) arg;
     FcitxKeyboard *keyboard = layout->owner;
-    keyboard->buffer[0] = '\0';
+    keyboard->cursor_moved = false;
+    keyboard->buffer[0][0] = '\0';
     keyboard->cursorPos = 0;
     keyboard->composeBuffer[0] = 0;
     keyboard->n_compose = 0;
@@ -539,7 +593,8 @@ void FcitxKeyboardOnClose(void* arg, FcitxIMCloseEventType event)
     }
 }
 
-static boolean IsDictAvailable(FcitxKeyboard* keyboard)
+static boolean
+IsDictAvailable(FcitxKeyboard* keyboard)
 {
     return FcitxSpellDictAvailable(keyboard->owner, keyboard->dictLang, NULL);
 }
@@ -556,9 +611,10 @@ FcitxKeyboardSetBuff(FcitxKeyboard *keyboard, const char *str)
     int len = strlen(str);
     if (len > FCITX_KEYBOARD_MAX_BUFFER)
         len = FCITX_KEYBOARD_MAX_BUFFER;
-    memcpy(keyboard->buffer, str, len);
+    memcpy(keyboard->buffer[0], str, len);
     keyboard->cursorPos = len;
-    keyboard->buffer[len] = '\0';
+    keyboard->buffer[0][len] = '\0';
+    keyboard->cursor_moved = false;
 }
 
 static INPUT_RETURN_VALUE
@@ -573,49 +629,60 @@ FcitxKeyboardHandleFocus(FcitxKeyboard *keyboard, FcitxKeySym sym,
         return IRV_TO_PROCESS;
     FcitxGlobalConfig *fc = FcitxInstanceGetGlobalConfig(instance);
     if (FcitxHotkeyIsHotKey(sym, state, fc->nextWord)) {
-        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
-        cand_word = FcitxCandidateWordGetNext(cand_list, cand_word);
-        if (!cand_word) {
-            FcitxCandidateWordSetPage(cand_list, 0);
-            cand_word = FcitxCandidateWordGetFirst(cand_list);
+        if (!keyboard->cursor_moved) {
+            cand_word = FcitxCandidateWordGetCurrentWindow(cand_list);
         } else {
-            FcitxCandidateWordSetFocus(
-                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+            cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+            cand_word = FcitxCandidateWordGetNext(cand_list, cand_word);
+            if (!cand_word) {
+                FcitxCandidateWordSetPage(cand_list, 0);
+            } else {
+                FcitxCandidateWordSetFocus(
+                    cand_list, FcitxCandidateWordGetIndex(cand_list,
+                                                          cand_word));
+            }
         }
     } else if (FcitxHotkeyIsHotKey(sym, state, fc->prevWord)) {
-        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
-        cand_word = FcitxCandidateWordGetPrev(cand_list, cand_word);
-        if (!cand_word) {
-            FcitxCandidateWordSetPage(
-                cand_list, FcitxCandidateWordPageCount(cand_list) - 1);
-            cand_word = FcitxCandidateWordGetLast(cand_list);
+        if (!keyboard->cursor_moved) {
+            cand_word = FcitxCandidateWordGetByIndex(
+                cand_list,
+                FcitxCandidateWordGetCurrentWindowSize(cand_list) - 1);
         } else {
-            FcitxCandidateWordSetFocus(
-                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
-        }
-    } else if (FcitxHotkeyIsHotKey(sym, state,
-                                   FcitxConfigPrevPageKey(instance, fc))) {
-        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
-        if (FcitxCandidateWordGoPrevPage(cand_list)) {
-            cand_word = FcitxCandidateWordGetCurrentWindow(cand_list) +
-                FcitxCandidateWordGetCurrentWindowSize(cand_list) - 1;
-        }
-    } else if (FcitxHotkeyIsHotKey(sym, state,
-                                   FcitxConfigNextPageKey(instance, fc))) {
-        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
-        if (FcitxCandidateWordGoNextPage(cand_list)) {
-            cand_word = FcitxCandidateWordGetCurrentWindow(cand_list);
+            cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+            cand_word = FcitxCandidateWordGetPrev(cand_list, cand_word);
+            if (cand_word) {
+                FcitxCandidateWordSetFocus(
+                    cand_list, FcitxCandidateWordGetIndex(cand_list,
+                                                          cand_word));
+            }
         }
     } else {
         return IRV_TO_PROCESS;
     }
-    FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
-    FcitxKeyboardSetBuff(keyboard, cand_word->strWord);
+    if (cand_word) {
+        FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+        if (!keyboard->cursor_moved)
+            memcpy(keyboard->buffer[1], keyboard->buffer[0],
+                   sizeof(keyboard->buffer[0]));
+        FcitxKeyboardSetBuff(keyboard, cand_word->strWord);
+        keyboard->cursor_moved = true;
+    } else if (keyboard->cursor_moved) {
+        FcitxKeyboardSetBuff(keyboard, keyboard->buffer[1]);
+    } else {
+        return IRV_FLAG_UPDATE_INPUT_WINDOW;
+    }
+    FcitxInputStateSetShowCursor(input, true);
+    FcitxInstanceCleanInputWindowUp(instance);
     FcitxMessages *client_preedit = FcitxInputStateGetClientPreedit(input);
-    FcitxMessagesSetMessageCount(client_preedit, 0);
     FcitxMessagesAddMessageStringsAtLast(client_preedit, MSG_INPUT,
-                                         keyboard->buffer);
+                                         keyboard->buffer[0]);
     FcitxInputStateSetClientCursorPos(input, keyboard->cursorPos);
+
+    if (!FcitxInstanceICSupportPreedit(instance, FcitxInstanceGetCurrentIC(instance))) {
+        FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetPreedit(input),
+                                             MSG_INPUT, keyboard->buffer[0]);
+        FcitxInputStateSetCursorPos(input, keyboard->cursorPos);
+    }
     return IRV_FLAG_UPDATE_INPUT_WINDOW;
 }
 
@@ -659,7 +726,7 @@ INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int
 
         if (FcitxHotkeyIsHotKey(sym, state,
                                 keyboard->config.hkAddToUserDict)) {
-            if (FcitxSpellAddPersonal(instance, keyboard->buffer,
+            if (FcitxSpellAddPersonal(instance, keyboard->buffer[0],
                                       keyboard->dictLang)) {
                 return IRV_DO_NOTHING;
             }
@@ -669,7 +736,7 @@ INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int
             IsValidSym(sym, state)) {
             if (IsValidChar(result) || FcitxHotkeyIsHotKeyLAZ(sym, state) ||
                 FcitxHotkeyIsHotKeyUAZ(sym, state) || IsValidSym(sym, state) ||
-                (*keyboard->buffer &&
+                (*keyboard->buffer[0] &&
                  FcitxHotkeyIsHotKey(sym, state, FCITX_HYPHEN_APOS))) {
                 char buf[UTF8_MAX_LENGTH + 1];
                 memset(buf, 0, sizeof(buf));
@@ -679,35 +746,36 @@ INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int
                     Ucs4ToUtf8(keyboard->iconv, FcitxKeySymToUnicode(sym), buf);
                 size_t charlen = strlen(buf);
 
-                size_t len = strlen(keyboard->buffer);
+                size_t len = strlen(keyboard->buffer[0]);
                 if (len >= FCITX_KEYBOARD_MAX_BUFFER) {
                     FcitxInstanceCommitString(instance, currentIC,
-                                              keyboard->buffer);
+                                              keyboard->buffer[0]);
                     keyboard->cursorPos = 0;
-                    keyboard->buffer[0] = '\0';
+                    keyboard->buffer[0][0] = '\0';
                     len = 0;
                 }
-                if (keyboard->buffer[keyboard->cursorPos] != 0) {
-                    memmove(keyboard->buffer + keyboard->cursorPos + charlen,
-                            keyboard->buffer + keyboard->cursorPos,
+                if (keyboard->buffer[0][keyboard->cursorPos] != 0) {
+                    memmove(keyboard->buffer[0] + keyboard->cursorPos + charlen,
+                            keyboard->buffer[0] + keyboard->cursorPos,
                             len - keyboard->cursorPos);
                 }
-                keyboard->buffer[len + charlen] = 0;
-                strncpy(&keyboard->buffer[keyboard->cursorPos], buf, charlen);
+                keyboard->buffer[0][len + charlen] = '\0';
+                strncpy(&keyboard->buffer[0][keyboard->cursorPos],
+                        buf, charlen);
                 keyboard->cursorPos += charlen;
 
                 return IRV_DISPLAY_CANDWORDS;
             }
         } else {
             if (FcitxHotkeyIsHotKey(sym, state, FCITX_BACKSPACE)) {
-                size_t slen = strlen(keyboard->buffer);
+                size_t slen = strlen(keyboard->buffer[0]);
                 if (slen > 0) {
                     if (keyboard->cursorPos > 0) {
-                        size_t len = fcitx_utf8_strlen(keyboard->buffer);
-                        char *pos = fcitx_utf8_get_nth_char(keyboard->buffer,
+                        size_t len = fcitx_utf8_strlen(keyboard->buffer[0]);
+                        char *pos = fcitx_utf8_get_nth_char(keyboard->buffer[0],
                                                             len - 1);
-                        keyboard->cursorPos = pos - keyboard->buffer;
-                        memset(keyboard->buffer + keyboard->cursorPos,
+                        keyboard->cursorPos = pos - keyboard->buffer[0];
+                        memset(keyboard->buffer[0] + keyboard->cursorPos,
                                0, sizeof(char) * (slen - keyboard->cursorPos));
                         return IRV_DISPLAY_CANDWORDS;
                     } else {
@@ -724,13 +792,13 @@ INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int
                 return IRV_TO_PROCESS;
         }
 
-        if (strlen(keyboard->buffer) > 0) {
+        if (strlen(keyboard->buffer[0]) > 0) {
             INPUT_RETURN_VALUE irv = IRV_FLAG_FORWARD_KEY;
             if (keyboard->config.bUseEnterToCommit &&
                 FcitxHotkeyIsHotKey(sym, state, FCITX_ENTER)) {
                 irv = 0;
             }
-            FcitxInstanceCommitString(instance, currentIC, keyboard->buffer);
+            FcitxInstanceCommitString(instance, currentIC, keyboard->buffer[0]);
 
             if (result) {
                 char buf[UTF8_MAX_LENGTH + 1];
@@ -776,7 +844,8 @@ INPUT_RETURN_VALUE FcitxKeyboardGetCandWords(void* arg)
     FcitxKeyboard* keyboard = layout->owner;
     FcitxInstance* instance = keyboard->owner;
     FcitxInputState* input = FcitxInstanceGetInputState(instance);
-    if (keyboard->buffer[0] == '\0')
+    keyboard->cursor_moved = false;
+    if (keyboard->buffer[0][0] == '\0')
         return IRV_CLEAN;
 
     static const unsigned int cmodtable[] = {
@@ -788,16 +857,17 @@ INPUT_RETURN_VALUE FcitxKeyboardGetCandWords(void* arg)
     FcitxCandidateWordSetPageSize(mainList, keyboard->config.maximumHintLength);
     FcitxCandidateWordSetChooseAndModifier(
         mainList, DIGIT_STR_CHOOSE, cmodtable[keyboard->config.chooseModifier]);
-    size_t bufferlen = strlen(keyboard->buffer);
+    size_t bufferlen = strlen(keyboard->buffer[0]);
     memcpy(FcitxInputStateGetRawInputBuffer(input),
-           keyboard->buffer, bufferlen + 1);
+           keyboard->buffer[0], bufferlen + 1);
     FcitxInputStateSetRawInputBufferSize(input, bufferlen);
+    FcitxInputStateSetShowCursor(input, true);
     FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetClientPreedit(input),
-                                         MSG_INPUT, keyboard->buffer);
+                                         MSG_INPUT, keyboard->buffer[0]);
     FcitxInputStateSetClientCursorPos(input, keyboard->cursorPos);
     if (!FcitxInstanceICSupportPreedit(instance, FcitxInstanceGetCurrentIC(instance))) {
         FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetPreedit(input),
-                                             MSG_INPUT, keyboard->buffer);
+                                             MSG_INPUT, keyboard->buffer[0]);
         FcitxInputStateSetCursorPos(input, keyboard->cursorPos);
     }
 
@@ -805,7 +875,7 @@ INPUT_RETURN_VALUE FcitxKeyboardGetCandWords(void* arg)
         return IRV_DISPLAY_CANDWORDS;
 
     FcitxCandidateWordList *newList;
-    newList = FcitxSpellGetCandWords(instance, NULL, keyboard->buffer, NULL,
+    newList = FcitxSpellGetCandWords(instance, NULL, keyboard->buffer[0], NULL,
                                      keyboard->config.maximumHintLength,
                                      keyboard->dictLang, NULL,
                                      FcitxKeyboardGetCandWordCb, layout);
@@ -822,8 +892,8 @@ void  FcitxKeyboardSave(void *arg)
 
 void  FcitxKeyboardReloadConfig(void *arg)
 {
-    FcitxKeyboardLayout* layout = (FcitxKeyboardLayout*) arg;
-    LoadKeyboardConfig(layout->owner, &layout->owner->config);
+    FcitxKeyboard* keyboard = (FcitxKeyboard*) arg;
+    LoadKeyboardConfig(keyboard, &keyboard->config);
 }
 
 boolean LoadKeyboardConfig(FcitxKeyboard* keyboard, FcitxKeyboardConfig* fs)
@@ -860,23 +930,35 @@ void SaveKeyboardConfig(FcitxKeyboardConfig* fs)
 
 INPUT_RETURN_VALUE FcitxKeyboardHotkeyToggleWordHint(void* arg)
 {
-    FcitxKeyboard* keyboard = (FcitxKeyboard*) arg;
-    FcitxIM* im = FcitxInstanceGetCurrentIM(keyboard->owner);
-    FcitxInputContext* currentIC = FcitxInstanceGetCurrentIC(keyboard->owner);
+    FcitxKeyboard *keyboard = (FcitxKeyboard*)arg;
+    FcitxInstance *instance = keyboard->owner;
+    FcitxIM *im = FcitxInstanceGetCurrentIM(instance);
+    FcitxInputContext *currentIC = FcitxInstanceGetCurrentIC(instance);
     if (!currentIC)
         return IRV_TO_PROCESS;
     if (im && strncmp(im->uniqueName, "fcitx-keyboard",
                       strlen("fcitx-keyboard")) == 0) {
-        void* enableWordHint = FcitxInstanceGetICData(keyboard->owner,
-                                                      currentIC,
-                                                      keyboard->dataSlot);
+        void *enableWordHint =
+            FcitxInstanceGetICData(instance, currentIC, keyboard->dataSlot);
+        boolean need_notify = true;
         if (!enableWordHint) {
             enableWordHint = PTR_TRUE;
-            IsDictAvailable(keyboard);
+            // also for preload dictionaries
+            need_notify = IsDictAvailable(keyboard);
         } else {
             enableWordHint = PTR_FALSE;
         }
-        FcitxInstanceSetICData(keyboard->owner, currentIC,
+
+        if (need_notify) {
+            FcitxFreeDesktopNotifyShowAddonTip(
+                instance, "fcitx-keyboard-hint",
+                "tools-check-spelling",
+                _("Spell hint"),
+                enableWordHint != PTR_FALSE ? _("Spell hint is enabled.") :
+                                              _("Spell hint is disabled."));
+        }
+
+        FcitxInstanceSetICData(instance, currentIC,
                                keyboard->dataSlot, enableWordHint);
         return IRV_DO_NOTHING;
     } else {
@@ -2800,7 +2882,9 @@ checkAlgorithmically(FcitxKeyboardLayout* layout)
     if (keyboard->n_compose >= FCITX_MAX_COMPOSE_LEN)
         return 0;
 
-    for (i = 0; i < keyboard->n_compose && IS_DEAD_KEY(keyboard->composeBuffer[i]); i++);
+    for (i = 0;i < keyboard->n_compose &&
+             IS_DEAD_KEY(keyboard->composeBuffer[i]);i++) {
+    }
     if (i == keyboard->n_compose)
         return INVALID_COMPOSE_RESULT;
 
