@@ -53,11 +53,6 @@
 #define PY_INDEX_MAGIC_NUMBER 0xf7462e34
 #define PINYIN_TEMP_FILE "pinyin_XXXXXX"
 
-FCITX_DEFINE_PLUGIN(fcitx_pinyin, ime, FcitxIMClass) = {
-    PYCreate,
-    PYDestroy
-};
-
 typedef struct {
     PY_CAND_WORD_TYPE type;
     ADJUSTORDER order;
@@ -72,6 +67,17 @@ static int PYCandWordCmp(const void* b, const void* a, void* arg);
 static boolean PYGetPYMapByHZ(FcitxPinyinState*pystate, char *strHZ,
                               char* mapHint, char *strMap);
 
+FCITX_DEFINE_PLUGIN(fcitx_pinyin, ime2, FcitxIMClass2) = {
+    PYCreate,
+    PYDestroy,
+    ReloadConfigPY,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
 DECLARE_ADDFUNCTIONS(Pinyin)
 
 void *PYCreate(FcitxInstance* instance)
@@ -105,7 +111,7 @@ void *PYCreate(FcitxInstance* instance)
                             PYGetCandWords,
                             NULL,
                             SavePY,
-                            ReloadConfigPY,
+                            NULL,
                             NULL,
                             5,
                             "zh_CN");
@@ -120,7 +126,7 @@ void *PYCreate(FcitxInstance* instance)
                             PYGetCandWords,
                             NULL,
                             SavePY,
-                            ReloadConfigPY,
+                            NULL,
                             NULL,
                             5,
                             "zh_CN");
@@ -391,19 +397,17 @@ boolean LoadPYOtherDict(FcitxPinyinState* pystate)
         FcitxStringHashSet *sset = FcitxXDGGetFiles("pinyin", NULL, ".mb");
         FcitxStringHashSet *curStr = sset;
         while (curStr) {
-            char *filename;
-
             if (strcmp(curStr->name, PY_PHRASE_FILE) != 0
                 && strcmp(curStr->name, PY_USERPHRASE_FILE) != 0
                 && strcmp(curStr->name, PY_SYMBOL_FILE) != 0
                 && strcmp(curStr->name, PY_BASE_FILE) != 0
                 && strcmp(curStr->name, PY_FREQ_FILE) != 0) {
 
-                fp = FcitxXDGGetFileWithPrefix("pinyin", curStr->name, "r", &filename);
-                FcitxLog(INFO, _("Load extra dict: %s"), filename);
-                free(filename);
-                LoadPYPhraseDict(pystate, fp, true, true);
-                fclose(fp);
+                fp = FcitxXDGGetFileWithPrefix("pinyin", curStr->name, "r", NULL);
+                if (fp) {
+                    LoadPYPhraseDict(pystate, fp, true, true);
+                    fclose(fp);
+                }
             }
             curStr = curStr->hh.next;
         }
@@ -688,6 +692,23 @@ INPUT_RETURN_VALUE DoPYInput(void* arg, FcitxKeySym sym, unsigned int state)
             }
 
             retVal = FcitxCandidateWordChooseByIndex(candList, 0);
+        } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ENTER)) {
+            if (FcitxInputStateGetRawInputBufferSize(input)) {
+                char* outputString = FcitxInputStateGetOutputString(input);
+                strcpy(outputString, "");
+                if (pystate->iPYSelected) {
+                    for (i = 0; i < pystate->iPYSelected; i++) {
+                        strcat(outputString, pystate->pySelected[i].strHZ);
+                    }
+                }
+
+                for (i = 0; i < pystate->findMap.iHZCount; i++) {
+                    strcat(outputString, pystate->findMap.strPYParsed[i]);
+                }
+                return IRV_COMMIT_STRING;
+            } else {
+                return IRV_TO_PROCESS;
+            }
         } else if (FcitxHotkeyIsHotKey(sym, state, pystate->pyconfig.hkPYDelUserPhr)) {
             if (!pystate->bIsPYDelUserPhr) {
                 int i;
@@ -1742,10 +1763,11 @@ void SavePYFreq(FcitxPinyinState *pystate)
 
     fclose(fp);
 
-    fp = FcitxXDGGetFileUserWithPrefix("pinyin", PY_FREQ_FILE, NULL, &pstr);
+    FcitxXDGGetFileUserWithPrefix("pinyin", PY_FREQ_FILE, NULL, &pstr);
     if (access(pstr, 0))
         unlink(pstr);
     rename(tempfile, pstr);
+
     free(pstr);
     free(tempfile);
     pystate->iNewFreqCount = 0;
@@ -1815,7 +1837,7 @@ void SavePYIndex(FcitxPinyinState *pystate)
 
     fclose(fp);
 
-    fp = FcitxXDGGetFileUserWithPrefix("pinyin", PY_INDEX_FILE, NULL, &pstr);
+    FcitxXDGGetFileUserWithPrefix("pinyin", PY_INDEX_FILE, NULL, &pstr);
     if (access(pstr, 0))
         unlink(pstr);
     rename(tempfile, pstr);
