@@ -91,10 +91,7 @@ boolean LoadTableConfig(TableConfig* config)
         return false;
 
     FILE *fp;
-    char *file;
-    fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-table.config", "r", &file);
-    FcitxLog(DEBUG, "Load Config File %s", file);
-    free(file);
+    fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-table.config", "r", NULL);
     if (!fp) {
         if (errno == ENOENT)
             SaveTableConfig(config);
@@ -487,7 +484,11 @@ INPUT_RETURN_VALUE DoTableInput(void* arg, FcitxKeySym sym, unsigned int state)
 
                         retVal = TableGetCandWords(table);
                         int key = FcitxInputStateGetRawInputBuffer(input)[0];
-                        if (!table->bIgnorePunc) {
+                        if (!table->bIgnorePunc
+                            || (table->bIgnorePunc
+                                && table->ignorePuncList
+                                && table->ignorePuncList[0]
+                                && strchr(table->ignorePuncList, key))) {
                             strTemp = FcitxPuncGetPunc(instance, &key);
                         } else {
                             strTemp = NULL;
@@ -528,8 +529,10 @@ INPUT_RETURN_VALUE DoTableInput(void* arg, FcitxKeySym sym, unsigned int state)
                              * 如果第一个字母是标点，并且没有候选字/词
                              * 则当做标点处理──适用于二笔这样的输入
                              **/
-                            strcpy(output_str, strTemp);
-                            retVal = IRV_PUNC;
+                            FcitxInputStateGetRawInputBuffer(input)[0] = '\0';
+                            FcitxInputStateSetRawInputBufferSize(input, 0);
+                            TableResetStatus(table);
+                            return IRV_TO_PROCESS;
                         }
                     }
                 } else {
