@@ -102,7 +102,7 @@ void Usage()
            "\t-d\t\t\trun as daemon(default)\n"
            "\t-D\t\t\tdon't run as daemon\n"
            "\t-s[sleep time]\t\toverride delay start time in config file, 0 for immediate start\n"
-           "\t-v\t\t\tdisplay the version information and exit\n"
+           "\t-v, --version\t\tdisplay the version information and exit\n"
            "\t-u, --ui\t\tspecify the user interface to use\n"
            "\t--enable\t\tspecify a comma separated list for addon that will override the enable option\n"
            "\t--disable\t\tspecify a comma separated list for addon that will explicitly disabled,\n"
@@ -124,10 +124,102 @@ FcitxInstance* FcitxInstanceCreate(sem_t *sem, int argc, char* argv[])
     return FcitxInstanceCreateWithFD(sem, argc, argv, -1);
 }
 
+FCITX_EXPORT_API
+boolean FcitxInstanceRun(int argc, char* argv[], int fd)
+{
+    FcitxInstance* instance = fcitx_utils_new(FcitxInstance);
+
+    do {
+        if (!ProcessOption(instance, argc, argv))
+            break;
+
+        instance->fd = fd;
+
+        RunInstance(instance);
+    } while(0);
+    boolean result = instance->loadingFatalError;
+//    free(instance);
+
+    return result;
+}
+
 FCITX_EXPORT_API
 FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int fd)
 {
-    FcitxInstance* instance = fcitx_utils_malloc0(sizeof(FcitxInstance));
+    if (!sem) {
+        return NULL;
+    }
+
+    FcitxInstance* instance = fcitx_utils_new(FcitxInstance);
+
+    if (!ProcessOption(instance, argc, argv))
+        goto create_error_exit_1;
+
+    instance->sem = sem;
+    instance->fd = fd;
+
+    if (sem_init(&instance->startUpSem, 0, 0) != 0) {
+        goto create_error_exit_1;
+    }
+
+    if (sem_init(&instance->notifySem, 0, 0) != 0) {
+        goto create_error_exit_2;
+    }
+
+    if (pthread_create(&instance->pid, NULL, RunInstance, instance) != 0) {
+        goto create_error_exit_3;
+    }
+
+    sem_wait(&instance->notifySem);
+
+    return instance;
+
+create_error_exit_3:
+    sem_destroy(&instance->notifySem);
+create_error_exit_2:
+    sem_destroy(&instance->startUpSem);
+create_error_exit_1:
+    free(instance);
+    return NULL;
+}
+
+FCITX_EXPORT_API
+void FcitxInstanceStart(FcitxInstance* instance)
+{
+    if (!instance->loadingFatalError) {
+        instance->initialized = true;
+
+        if (sem_post(&instance->startUpSem))
+            instance->initialized = false;
+    }
+}
+
+
+FCITX_EXPORT_API
+FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int fd)
+{
+    FcitxInstance* instance = FcitxInstanceCreatePause(sem, argc, argv, fd);
+
+    if (instance) {
+        FcitxInstanceStart(instance);
+    }
+
+    return instance;
+
+}
+
+FCITX_EXPORT_API
+void FcitxInstanceSetRecheckEvent(FcitxInstance* instance)
+{
+    instance->eventflag |= FEF_EVENT_CHECK;
+}
+
+FCITX_EXPORT_API
+jmp_buf FcitxRecover;
+
+void* RunInstance(void* arg)
+{
+    FcitxInstance* instance = (FcitxInstance*) arg;
     FcitxAddonsInit(&instance->addons);
     FcitxInstanceInitIM(instance);
     FcitxFrontendsInit(&instance->frontends);
@@ -139,14 +231,13 @@ FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int
     utarray_init(&instance->timeout, &timeout_icd);
     utarray_init(&instance->icdata, &icdata_icd);
     instance->input = FcitxInputStateCreate();
-    instance->sem = sem;
     instance->config = fcitx_utils_malloc0(sizeof(FcitxGlobalConfig));
     instance->profile = fcitx_utils_malloc0(sizeof(FcitxProfile));
     instance->globalIMName = strdup("");
-    instance->fd = -1;
-    if (fd >= 0) {
-        fcntl(fd, F_SETFL, O_NONBLOCK);
-        instance->fd = fd;
+    if (instance->fd >= 0) {
+        fcntl(instance->fd, F_SETFL, O_NONBLOCK);
+    } else {
+        instance->fd = -1;
     }
 
     if (!FcitxGlobalConfigLoad(instance->config))
@@ -154,8 +245,13 @@ FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int
 
     FcitxCandidateWordSetPageSize(instance->input->candList, instance->config->iMaxCandWord);
 
-    if (!ProcessOption(instance, argc, argv))
-        goto error_exit;
+    int overrideDelay = instance->overrideDelay;
+
+    if (overrideDelay < 0)
+        overrideDelay = instance->config->iDelayStart;
+
+    if (overrideDelay > 0)
+        sleep(overrideDelay);
 
     instance->timeStart = time(NULL);
     instance->globalState = instance->config->defaultIMState;
@@ -176,10 +272,9 @@ FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int
     FcitxInstanceInitBuiltContext(instance);
     FcitxModuleLoad(instance);
     if (instance->loadingFatalError)
-        return instance;
+        return NULL;
     if (!FcitxInstanceLoadAllIM(instance)) {
-        FcitxInstanceEnd(instance);
-        return instance;
+        goto error_exit;
     }
 
     FcitxInstanceInitIMMenu(instance);
@@ -196,52 +291,17 @@ FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int
     FcitxInstanceSwitchIMByIndex(instance, instance->iIMIndex);
 
     if (!FcitxInstanceLoadFrontend(instance)) {
-        FcitxInstanceEnd(instance);
-        return instance;
+        goto error_exit;
     }
 
-    return instance;
-
-error_exit:
-    FcitxInstanceEnd(instance);
-    return instance;
-}
-
-FCITX_EXPORT_API
-void FcitxInstanceStart(FcitxInstance* instance)
-{
-    if (!instance->loadingFatalError) {
+    /* fcitx is running in a standalone thread or not */
+    if (instance->sem) {
+        sem_post(&instance->notifySem);
+        sem_wait(&instance->startUpSem);
+    } else {
         instance->initialized = true;
-        /* make in order to use block X, query is not good here */
-        if (pthread_create(&instance->pid, NULL, RunInstance, instance) != 0)
-            instance->initialized = false;
     }
-}
 
-
-FCITX_EXPORT_API
-FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int fd)
-{
-    FcitxInstance* instance = FcitxInstanceCreatePause(sem, argc, argv, fd);
-
-    FcitxInstanceStart(instance);
-
-    return instance;
-
-}
-
-FCITX_EXPORT_API
-void FcitxInstanceSetRecheckEvent(FcitxInstance* instance)
-{
-    instance->uiflag |= UI_EVENT_CHECK;
-}
-
-FCITX_EXPORT_API
-jmp_buf FcitxRecover;
-
-void* RunInstance(void* arg)
-{
-    FcitxInstance* instance = (FcitxInstance*) arg;
     uint64_t curtime = 0;
     while (1) {
         FcitxAddon** pmodule;
@@ -257,10 +317,10 @@ void* RunInstance(void* arg)
             }
         }
         do {
-            instance->uiflag = UI_NONE;
+            instance->eventflag &= (~FEF_PROCESS_EVENT_MASK);
             for (pmodule = (FcitxAddon**) utarray_front(&instance->eventmodules);
-                    pmodule != NULL;
-                    pmodule = (FcitxAddon**) utarray_next(&instance->eventmodules, pmodule)) {
+                  pmodule != NULL;
+                  pmodule = (FcitxAddon**) utarray_next(&instance->eventmodules, pmodule)) {
                 FcitxModule* module = (*pmodule)->module;
                 module->ProcessEvent((*pmodule)->addonInstance);
             }
@@ -289,12 +349,12 @@ void* RunInstance(void* arg)
                 }
             }
 
-            if (instance->uiflag & UI_MOVE)
+            if (instance->eventflag & FEF_UI_MOVE)
                 FcitxUIMoveInputWindowReal(instance);
 
-            if (instance->uiflag & UI_UPDATE)
+            if (instance->eventflag & FEF_UI_UPDATE)
                 FcitxUIUpdateInputWindowReal(instance);
-        } while (instance->uiflag != UI_NONE);
+        } while ((instance->eventflag & FEF_PROCESS_EVENT_MASK) != FEF_NONE);
 
         setjmp(FcitxRecover);
 
@@ -302,6 +362,11 @@ void* RunInstance(void* arg)
             FcitxInstanceRealEnd(instance);
             break;
         }
+        
+        if (instance->eventflag & FEF_RELOAD_ADDON) {
+            instance->eventflag &= ~(FEF_RELOAD_ADDON);
+            FcitxInstanceReloadAddon(instance);
+        }
 
         FD_ZERO(&instance->rfds);
         FD_ZERO(&instance->wfds);
@@ -313,8 +378,8 @@ void* RunInstance(void* arg)
             FD_SET(instance->fd, &instance->rfds);
         }
         for (pmodule = (FcitxAddon**) utarray_front(&instance->eventmodules);
-                pmodule != NULL;
-                pmodule = (FcitxAddon**) utarray_next(&instance->eventmodules, pmodule)) {
+              pmodule != NULL;
+              pmodule = (FcitxAddon**) utarray_next(&instance->eventmodules, pmodule)) {
             FcitxModule* module = (*pmodule)->module;
             module->SetFD((*pmodule)->addonInstance);
         }
@@ -339,6 +404,11 @@ void* RunInstance(void* arg)
                &instance->efds, ptval);
     }
     return NULL;
+
+error_exit:
+    sem_post(&instance->startUpSem);
+    FcitxInstanceEnd(instance);
+    return NULL;
 }
 
 FCITX_EXPORT_API
@@ -353,7 +423,10 @@ void FcitxInstanceEnd(FcitxInstance* instance)
             if (!instance->quietQuit)
                 FcitxLog(ERROR, "Exiting.");
             instance->loadingFatalError = true;
-            sem_post(instance->sem);
+
+            if (instance->sem) {
+                sem_post(instance->sem);
+            }
         }
         return;
     }
@@ -416,9 +489,9 @@ void FcitxInstanceRealEnd(FcitxInstance* instance) {
     }
 
     FcitxAddon** pmodule;
-    for (pmodule = (FcitxAddon**) utarray_front(&instance->modules);
+    for (pmodule = (FcitxAddon**) utarray_back(&instance->modules);
          pmodule != NULL;
-         pmodule = (FcitxAddon**) utarray_next(&instance->modules, pmodule)) {
+         pmodule = (FcitxAddon**) utarray_prev(&instance->modules, pmodule)) {
         if (pmodule == NULL)
             return;
         FcitxModule* module = (*pmodule)->module;
@@ -426,7 +499,9 @@ void FcitxInstanceRealEnd(FcitxInstance* instance) {
             module->Destroy((*pmodule)->addonInstance);
     }
 
-    sem_post(instance->sem);
+    if (instance->sem) {
+        sem_post(instance->sem);
+    }
 }
 
 void FcitxInitThread(FcitxInstance* inst)
@@ -476,6 +551,7 @@ boolean ProcessOption(FcitxInstance* instance, int argc, char* argv[])
         {"replace", 0, 0, 0},
         {"enable", 1, 0, 0},
         {"disable", 1, 0, 0},
+        {"version", 0, 0, 0},
         {"help", 0, 0, 0},
         {NULL, 0, 0, 0}
     };
@@ -509,6 +585,11 @@ boolean ProcessOption(FcitxInstance* instance, int argc, char* argv[])
                     instance->disableList = fcitx_utils_split_string(optarg, ',');
                 }
                 break;
+            case 4:
+                instance->quietQuit = true;
+                Version();
+                return false;
+                break;
             default:
                 instance->quietQuit = true;
                 Usage();
@@ -532,13 +613,16 @@ boolean ProcessOption(FcitxInstance* instance, int argc, char* argv[])
             overrideDelay = atoi(optarg);
             break;
         case 'h':
+            instance->quietQuit = true;
             Usage();
             return false;
         case 'v':
+            instance->quietQuit = true;
             Version();
             return false;
             break;
         default:
+            instance->quietQuit = true;
             Usage();
             return false;
         }
@@ -552,11 +636,7 @@ boolean ProcessOption(FcitxInstance* instance, int argc, char* argv[])
     if (runasdaemon)
         fcitx_utils_init_as_daemon();
 
-    if (overrideDelay < 0)
-        overrideDelay = instance->config->iDelayStart;
-
-    if (overrideDelay > 0)
-        sleep(overrideDelay);
+    instance->overrideDelay = overrideDelay;
 
     return true;
 }
@@ -640,11 +720,7 @@ uint64_t FcitxInstanceAddTimeout(FcitxInstance* instance, long int milli, FcitxT
 FCITX_EXPORT_API
 boolean FcitxInstanceCheckTimeoutByFunc(FcitxInstance* instance, FcitxTimeoutCallback callback)
 {
-    TimeoutItem* ti;
-    for (ti = (TimeoutItem*) utarray_front(&instance->timeout);
-         ti != NULL;
-         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti))
-    {
+    utarray_foreach(ti, &instance->timeout, TimeoutItem) {
         if (ti->callback == callback)
             return true;
     }
@@ -654,9 +730,7 @@ boolean FcitxInstanceCheckTimeoutByFunc(FcitxInstance* instance, FcitxTimeoutCal
 FCITX_EXPORT_API
 boolean FcitxInstanceCheckTimeoutById(FcitxInstance *instance, uint64_t id)
 {
-    TimeoutItem *ti;
-    for (ti = (TimeoutItem*)utarray_front(&instance->timeout);ti;
-         ti = (TimeoutItem*)utarray_next(&instance->timeout, ti)) {
+    utarray_foreach(ti, &instance->timeout, TimeoutItem) {
         if (ti->idx == id)
             return true;
     }
@@ -667,10 +741,7 @@ FCITX_EXPORT_API boolean
 FcitxInstanceRemoveTimeoutByFunc(FcitxInstance* instance,
                                  FcitxTimeoutCallback callback)
 {
-    TimeoutItem* ti;
-    for (ti = (TimeoutItem*) utarray_front(&instance->timeout);
-         ti != NULL;
-         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti)) {
+    utarray_foreach(ti, &instance->timeout, TimeoutItem) {
         if (ti->callback == callback) {
             unsigned int idx = utarray_eltidx(&instance->timeout, ti);
             utarray_remove_quick(&instance->timeout, idx);
@@ -685,10 +756,7 @@ boolean FcitxInstanceRemoveTimeoutById(FcitxInstance* instance, uint64_t id)
 {
     if (id == 0)
         return false;
-    TimeoutItem* ti;
-    for (ti = (TimeoutItem*) utarray_front(&instance->timeout);
-         ti != NULL;
-         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti)) {
+    utarray_foreach(ti, &instance->timeout, TimeoutItem) {
         if (ti->idx == id) {
             unsigned int idx = utarray_eltidx(&instance->timeout, ti);
             utarray_remove_quick(&instance->timeout, idx);
@@ -700,10 +768,7 @@ boolean FcitxInstanceRemoveTimeoutById(FcitxInstance* instance, uint64_t id)
 
 FCITX_EXPORT_API
 int FcitxInstanceWaitForEnd(FcitxInstance* instance) {
-    if (instance->initialized) {
-        return pthread_join(instance->pid, NULL);
-    }
-    return 0;
+    return pthread_join(instance->pid, NULL);
 }
 
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
