@@ -45,69 +45,69 @@
 #define GetMenuItem(m, i) ((FcitxMenuItem*) utarray_eltptr(&(m)->shell, (i)))
 
 const char * kimpanel_introspection_xml =
-    "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n"
-    "\"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n"
-    "<node name=\"" FCITX_KIMPANEL_PATH "\">\n"
-    "  <interface name=\"org.freedesktop.DBus.Introspectable\">\n"
-    "    <method name=\"Introspect\">\n"
-    "      <arg name=\"data\" direction=\"out\" type=\"s\"/>\n"
-    "    </method>\n"
-    "  </interface>\n"
-    "  <interface name=\"" FCITX_KIMPANEL_INTERFACE "\">\n"
-    "    <signal name=\"ExecDialog\">\n"
-    "      <arg name=\"prop\" direction=\"in\" type=\"s\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"ExecMenu\">\n"
-    "      <arg name=\"prop\" direction=\"in\" type=\"a\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"RegisterProperties\">\n"
-    "      <arg name=\"prop\" direction=\"in\" type=\"a\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdateProperty\">\n"
-    "      <arg name=\"prop\" direction=\"in\" type=\"s\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"RemoveProperty\">\n"
-    "      <arg name=\"prop\" direction=\"in\" type=\"s\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"ShowAux\">\n"
-    "      <arg name=\"toshow\" direction=\"in\" type=\"b\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"ShowPreedit\">\n"
-    "      <arg name=\"toshow\" direction=\"in\" type=\"b\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"ShowLookupTable\">\n"
-    "      <arg name=\"toshow\" direction=\"in\" type=\"b\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdateLookupTable\">\n"
-    "      <arg name=\"label\" direction=\"in\" type=\"a\"/>\n"
-    "      <arg name=\"text\" direction=\"in\" type=\"a\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdateLookupTableCursor\">\n"
-    "      <arg name=\"cursor\" direction=\"in\" type=\"i\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdatePreeditCaret\">\n"
-    "      <arg name=\"position\" direction=\"in\" type=\"i\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdatePreeditText\">\n"
-    "      <arg name=\"text\" direction=\"in\" type=\"s\"/>\n"
-    "      <arg name=\"attr\" direction=\"in\" type=\"s\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdateAux\">\n"
-    "      <arg name=\"text\" direction=\"in\" type=\"s\"/>\n"
-    "      <arg name=\"attr\" direction=\"in\" type=\"s\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdateSpotLocation\">\n"
-    "      <arg name=\"x\" direction=\"in\" type=\"i\"/>\n"
-    "      <arg name=\"y\" direction=\"in\" type=\"i\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"UpdateScreen\">\n"
-    "      <arg name=\"screen\" direction=\"in\" type=\"i\"/>\n"
-    "    </signal>\n"
-    "    <signal name=\"Enable\">\n"
-    "      <arg name=\"toenable\" direction=\"in\" type=\"b\"/>\n"
-    "    </signal>\n"
-    "  </interface>\n"
-    "</node>\n";
+    "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\" "
+    "\"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">"
+    "<node name=\"" FCITX_KIMPANEL_PATH "\">"
+    "<interface name=\"org.freedesktop.DBus.Introspectable\">"
+    "<method name=\"Introspect\">"
+    "<arg name=\"data\" direction=\"out\" type=\"s\"/>"
+    "</method>"
+    "</interface>"
+    "<interface name=\"" FCITX_KIMPANEL_INTERFACE "\">"
+    "<signal name=\"ExecDialog\">"
+    "<arg name=\"prop\" direction=\"in\" type=\"s\"/>"
+    "</signal>"
+    "<signal name=\"ExecMenu\">"
+    "<arg name=\"prop\" direction=\"in\" type=\"as\"/>"
+    "</signal>"
+    "<signal name=\"RegisterProperties\">"
+    "<arg name=\"prop\" direction=\"in\" type=\"as\"/>"
+    "</signal>"
+    "<signal name=\"UpdateProperty\">"
+    "<arg name=\"prop\" direction=\"in\" type=\"s\"/>"
+    "</signal>"
+    "<signal name=\"RemoveProperty\">"
+    "<arg name=\"prop\" direction=\"in\" type=\"s\"/>"
+    "</signal>"
+    "<signal name=\"ShowAux\">"
+    "<arg name=\"toshow\" direction=\"in\" type=\"b\"/>"
+    "</signal>"
+    "<signal name=\"ShowPreedit\">"
+    "<arg name=\"toshow\" direction=\"in\" type=\"b\"/>"
+    "</signal>"
+    "<signal name=\"ShowLookupTable\">"
+    "<arg name=\"toshow\" direction=\"in\" type=\"b\"/>"
+    "</signal>"
+    "<signal name=\"UpdateLookupTable\">"
+    "<arg name=\"label\" direction=\"in\" type=\"as\"/>"
+    "<arg name=\"text\" direction=\"in\" type=\"as\"/>"
+    "</signal>"
+    "<signal name=\"UpdateLookupTableCursor\">"
+    "<arg name=\"cursor\" direction=\"in\" type=\"i\"/>"
+    "</signal>"
+    "<signal name=\"UpdatePreeditCaret\">"
+    "<arg name=\"position\" direction=\"in\" type=\"i\"/>"
+    "</signal>"
+    "<signal name=\"UpdatePreeditText\">"
+    "<arg name=\"text\" direction=\"in\" type=\"s\"/>"
+    "<arg name=\"attr\" direction=\"in\" type=\"s\"/>"
+    "</signal>"
+    "<signal name=\"UpdateAux\">"
+    "<arg name=\"text\" direction=\"in\" type=\"s\"/>"
+    "<arg name=\"attr\" direction=\"in\" type=\"s\"/>"
+    "</signal>"
+    "<signal name=\"UpdateSpotLocation\">"
+    "<arg name=\"x\" direction=\"in\" type=\"i\"/>"
+    "<arg name=\"y\" direction=\"in\" type=\"i\"/>"
+    "</signal>"
+    "<signal name=\"UpdateScreen\">"
+    "<arg name=\"screen\" direction=\"in\" type=\"i\"/>"
+    "</signal>"
+    "<signal name=\"Enable\">"
+    "<arg name=\"toenable\" direction=\"in\" type=\"b\"/>"
+    "</signal>"
+    "</interface>"
+    "</node>";
 
 typedef struct _FcitxKimpanelUI {
     FcitxInstance* owner;
@@ -142,6 +142,7 @@ static void KimpanelOnTriggerOn(void *arg);
 static void KimpanelOnTriggerOff(void *arg);
 static void KimpanelSuspend(void* arg);
 static void KimpanelDestroy(void* arg);
+static void KimpanelOwnerChanged(void* user_data, void* arg, const char* serviceName, const char* oldName, const char* newName);
 
 static void KimUpdateLookupTableCursor(FcitxKimpanelUI* kimpanel, int cursor);
 static void KimShowAux(FcitxKimpanelUI* kimpanel, boolean toShow);
@@ -166,7 +167,7 @@ static void KimUpdateProperty(FcitxKimpanelUI* kimpanel, char *prop);
 static DBusHandlerResult KimpanelDBusEventHandler(DBusConnection *connection, DBusMessage *message, void *user_data);
 static DBusHandlerResult KimpanelDBusFilter(DBusConnection *connection, DBusMessage *message, void *user_data);
 static int CalKimCursorPos(FcitxKimpanelUI *kimpanel);
-static void KimpanelInputReset(void *arg);
+static void KimpanelInputIMChanged(void *arg);
 static char* Status2String(FcitxUIStatus* status);
 static char* ComplexStatus2String(FcitxUIComplexStatus* status);
 static void KimpanelRegisterAllStatus(FcitxKimpanelUI* kimpanel);
@@ -177,6 +178,9 @@ static void KimpanelIntrospect(FcitxKimpanelUI* kimpanel);
 static void KimpanelIntrospectCallback(DBusPendingCall *call, void *data);
 
 #define KIMPANEL_BUFFER_SIZE 4096
+#ifndef DBUS_TIMEOUT_USE_DEFAULT
+#  define DBUS_TIMEOUT_USE_DEFAULT (-1)
+#endif
 
 FCITX_DEFINE_PLUGIN(fcitx_kimpanel_ui, ui, FcitxUI) = {
     KimpanelCreate,
@@ -292,7 +296,7 @@ void* KimpanelCreate(FcitxInstance* instance)
 
         // add a rule to receive signals from kimpanel
         dbus_bus_add_match(kimpanel->conn,
-                           "type='signal',interface='org.kde.impanel'",
+                           "type='signal',sender='org.kde.impanel',interface='org.kde.impanel'",
                            &err);
         dbus_connection_flush(kimpanel->conn);
         if (dbus_error_is_set(&err)) {
@@ -300,7 +304,7 @@ void* KimpanelCreate(FcitxInstance* instance)
             break;
         }
         dbus_bus_add_match(kimpanel->conn,
-                           "type='signal',interface='org.kde.impanel2'",
+                           "type='signal',sender='org.kde.impanel',interface='org.kde.impanel2'",
                            &err);
         dbus_connection_flush(kimpanel->conn);
         if (dbus_error_is_set(&err)) {
@@ -308,17 +312,9 @@ void* KimpanelCreate(FcitxInstance* instance)
             break;
         }
 
-        dbus_bus_add_match(kimpanel->conn,
-                           "type='signal',"
-                           "interface='" DBUS_INTERFACE_DBUS "',"
-                           "path='" DBUS_PATH_DBUS "',"
-                           "member='NameOwnerChanged',"
-                           "arg0='org.kde.impanel'",
-                           &err);
-
-        dbus_connection_flush(kimpanel->conn);
-        if (dbus_error_is_set(&err)) {
-            FcitxLog(ERROR, "Match Error (%s)", err.message);
+        int id = FcitxDBusWatchName(instance, "org.kde.impanel", kimpanel,
+                                    KimpanelOwnerChanged, NULL, NULL);
+        if (id == 0) {
             break;
         }
 
@@ -334,24 +330,25 @@ void* KimpanelCreate(FcitxInstance* instance)
         kimpanel->messageUp = FcitxMessagesNew();
         kimpanel->messageDown = FcitxMessagesNew();
 
-        FcitxIMEventHook resethk;
-        resethk.arg = kimpanel;
-        resethk.func = KimpanelInputReset;
-        FcitxInstanceRegisterResetInputHook(instance, resethk);
+        FcitxIMEventHook imchangehk;
+        imchangehk.arg = kimpanel;
+        imchangehk.func = KimpanelInputIMChanged;
+        FcitxInstanceRegisterIMChangedHook(instance, imchangehk);
 
         const char* kimpanelServiceName = "org.kde.impanel";
         DBusMessage* message = dbus_message_new_method_call(DBUS_SERVICE_DBUS, DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "NameHasOwner");
         dbus_message_append_args(message, DBUS_TYPE_STRING, &kimpanelServiceName, DBUS_TYPE_INVALID);
 
-        DBusPendingCall* call = NULL;
-        dbus_bool_t reply = dbus_connection_send_with_reply(kimpanel->conn, message,
-                                                            &call,
-                                                            0);
+        DBusPendingCall *call = NULL;
+        dbus_bool_t reply =
+            dbus_connection_send_with_reply(kimpanel->conn, message,
+                                            &call, DBUS_TIMEOUT_USE_DEFAULT);
         if (reply == TRUE) {
             dbus_pending_call_set_notify(call,
                                          KimpanelServiceExistCallback,
                                          kimpanel,
                                          NULL);
+            dbus_pending_call_unref(call);
         }
         dbus_connection_flush(kimpanel->conn);
         dbus_message_unref(message);
@@ -417,7 +414,7 @@ void KimpanelSetIMStatus(FcitxKimpanelUI* kimpanel)
     free(status);
 }
 
-void KimpanelInputReset(void* arg)
+void KimpanelInputIMChanged(void* arg)
 {
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
     if (kimpanel->addon != FcitxInstanceGetCurrentUI(kimpanel->owner))
@@ -875,33 +872,20 @@ DBusHandlerResult KimpanelDBusFilter(DBusConnection* connection, DBusMessage* ms
         FcitxLog(DEBUG, "Configure");
         fcitx_utils_launch_configure_tool();
         return DBUS_HANDLER_RESULT_HANDLED;
-    } else if (dbus_message_is_signal(msg, DBUS_INTERFACE_DBUS, "NameOwnerChanged")) {
-        const char* service, *oldowner, *newowner;
-        DBusError error;
-        dbus_error_init(&error);
-        if (dbus_message_get_args(msg, &error,
-                                  DBUS_TYPE_STRING, &service ,
-                                  DBUS_TYPE_STRING, &oldowner ,
-                                  DBUS_TYPE_STRING, &newowner ,
-                                  DBUS_TYPE_INVALID)) {
-            /* old die and no new one */
-            if (strcmp(service, "org.kde.impanel") == 0
-                && strlen(oldowner) > 0
-                && strlen(newowner) == 0)
-                FcitxUISwitchToFallback(instance);
-            /*
-             * since if new rise, it will send PanelCreated,
-             * So don't need to re-register here
-             */
-        }
-        dbus_error_free(&error);
-        return DBUS_HANDLER_RESULT_HANDLED;
     }
 
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
-
+void KimpanelOwnerChanged(void* user_data, void* arg, const char* serviceName, const char* oldName, const char* newName) {
+    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) user_data;
+    /* old die and no new one */
+    if (strcmp(serviceName, "org.kde.impanel") == 0) {
+        if (strlen(oldName) > 0 && strlen(newName) == 0) {
+            FcitxUISwitchToFallback(kimpanel->owner);
+        }
+    }
+}
 
 void KimExecDialog(FcitxKimpanelUI* kimpanel, char *prop)
 {
@@ -1275,7 +1259,8 @@ void KimUpdateLookupTable(FcitxKimpanelUI* kimpanel, char *labels[], int nLabel,
     dbus_message_iter_init_append(msg, &args);
     dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "s", &subLabel);
     for (i = 0; i < nLabel; i++) {
-        if (!dbus_message_iter_append_basic(&subLabel, DBUS_TYPE_STRING, &labels[i])) {
+        if (!dbus_message_iter_append_basic(&subLabel, DBUS_TYPE_STRING,
+                                            &labels[i])) {
             FcitxLog(DEBUG, "Out Of Memory!");
         }
     }
@@ -1283,7 +1268,8 @@ void KimUpdateLookupTable(FcitxKimpanelUI* kimpanel, char *labels[], int nLabel,
 
     dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "s", &subText);
     for (i = 0; i < nText; i++) {
-        if (!dbus_message_iter_append_basic(&subText, DBUS_TYPE_STRING, &texts[i])) {
+        if (!dbus_message_iter_append_basic(&subText, DBUS_TYPE_STRING,
+                                            &texts[i])) {
             FcitxLog(DEBUG, "Out Of Memory!");
         }
     }
@@ -1638,27 +1624,24 @@ int CalKimCursorPos(FcitxKimpanelUI *kimpanel)
 
 void KimpanelServiceExistCallback(DBusPendingCall *call, void *data)
 {
-    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) data;
+    FcitxKimpanelUI *kimpanel = (FcitxKimpanelUI*)data;
 
-    DBusMessage* msg = dbus_pending_call_steal_reply(call);
+    DBusMessage *msg = dbus_pending_call_steal_reply(call);
 
     if (msg) {
-        dbus_bool_t has;
+        dbus_bool_t has = FALSE;
         DBusError error;
         dbus_error_init(&error);
-        dbus_message_get_args(msg, &error, DBUS_TYPE_BOOLEAN, &has , DBUS_TYPE_INVALID);
-        if (!has) {
-            FcitxUISwitchToFallback(kimpanel->owner);
-        }
-        else {
-            KimpanelIntrospect(kimpanel);
-        }
+        dbus_message_get_args(msg, &error,
+                              DBUS_TYPE_BOOLEAN, &has, DBUS_TYPE_INVALID);
         dbus_message_unref(msg);
         dbus_error_free(&error);
+        if (!has) {
+            FcitxUISwitchToFallback(kimpanel->owner);
+        } else {
+            KimpanelIntrospect(kimpanel);
+        }
     }
-
-    dbus_pending_call_cancel(call);
-    dbus_pending_call_unref(call);
 }
 
 void KimpanelIntrospect(FcitxKimpanelUI* kimpanel)
@@ -1666,43 +1649,40 @@ void KimpanelIntrospect(FcitxKimpanelUI* kimpanel)
     const char* kimpanelServiceName = "org.kde.impanel";
     DBusMessage* message = dbus_message_new_method_call(kimpanelServiceName, "/org/kde/impanel", DBUS_INTERFACE_INTROSPECTABLE, "Introspect");
 
-    DBusPendingCall* call = NULL;
-    dbus_bool_t reply = dbus_connection_send_with_reply(kimpanel->conn, message,
-                                                        &call,
-                                                        0);
+    DBusPendingCall *call = NULL;
+    dbus_bool_t reply =
+        dbus_connection_send_with_reply(kimpanel->conn, message,
+                                        &call, DBUS_TIMEOUT_USE_DEFAULT);
+    dbus_message_unref(message);
     if (reply == TRUE) {
-        dbus_pending_call_set_notify(call,
-                                     KimpanelIntrospectCallback,
-                                     kimpanel,
-                                     NULL);
+        dbus_pending_call_set_notify(call, KimpanelIntrospectCallback,
+                                     kimpanel, NULL);
+        dbus_pending_call_unref(call);
     }
     dbus_connection_flush(kimpanel->conn);
-    dbus_message_unref(message);
 }
 
-void KimpanelIntrospectCallback(DBusPendingCall* call, void* data)
+void KimpanelIntrospectCallback(DBusPendingCall *call, void *data)
 {
-    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) data;
-
-    DBusMessage* msg = dbus_pending_call_steal_reply(call);
+    FcitxKimpanelUI *kimpanel = (FcitxKimpanelUI*)data;
+    DBusMessage *msg = dbus_pending_call_steal_reply(call);
 
     if (msg) {
-        const char* s;
+        const char *s;
         DBusError error;
         dbus_error_init(&error);
-        if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &s , DBUS_TYPE_INVALID)) {
+        if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &s,
+                                  DBUS_TYPE_INVALID)) {
             if (strstr(s, "org.kde.impanel2")) {
                 kimpanel->version = 2;
-                if (strstr(s, "SetLookupTable"))
+                if (strstr(s, "SetLookupTable")) {
                     kimpanel->hasSetLookupTable = true;
+                }
             }
         }
         dbus_message_unref(msg);
         dbus_error_free(&error);
     }
-
-    dbus_pending_call_cancel(call);
-    dbus_pending_call_unref(call);
 }
 
 
@@ -1723,16 +1703,10 @@ void KimpanelDestroy(void* arg)
     dbus_connection_remove_filter(kimpanel->conn, KimpanelDBusFilter, kimpanel);
 
     dbus_bus_remove_match(kimpanel->conn,
-                          "type='signal',interface='org.kde.impanel'",
-                          NULL);
-    dbus_connection_flush(kimpanel->conn);
-
+                          "type='signal',sender='org.kde.impanel',interface='org.kde.impanel'",
+                           NULL);
     dbus_bus_remove_match(kimpanel->conn,
-                          "type='signal',"
-                          "interface='" DBUS_INTERFACE_DBUS "',"
-                          "path='" DBUS_PATH_DBUS "',"
-                          "member='NameOwnerChanged',"
-                          "arg0='org.kde.impanel'",
+                          "type='signal',sender='org.kde.impanel',interface='org.kde.impanel2'",
                           NULL);
 
     dbus_connection_flush(kimpanel->conn);
